

var startViewId = null;
var viewsWithIds = {};
var parentViewsWithIds = {};
var targetViews = [];
var importViews = [];
var textPropertyViews = [];
var isViewController = true;

function startWithViewId(jsonText,viewId,theIsViewController) {

  startViewId = viewId;
  isViewController = theIsViewController;
  viewsWithIds = {};
  parentViewsWithIds = {};
  targetViews = [];
  importViews = [];
  textPropertyViews = [];
  imagePropertyViews = [];

  var data = JSON.parse(jsonText);


  parseViews(data,false);

  return output();
}

if (module) {
  module.exports = startWithViewId;
}

function parseViews(view,isInTarget) {

  if (view.originalId == startViewId) {
    isInTarget = true
  }
  if (isInTarget) {
    targetViews.push(view);

    if (view.type == 'Label' && !isViewController && view.originalId != startViewId) {
      textPropertyViews.push(view);
    }

    if (view.type == 'ImageView' && !isViewController && view.originalId != startViewId) {
      imagePropertyViews.push(view);
    }

  }

  if (view.children) {
    for(var i=0; i<view.children.length; i++) {
      var child = view.children[i]
      parentViewsWithIds[child.originalId] = view;
      viewsWithIds[child.originalId] = child;

      if (child.writeClassType && child.writeClassType == 'OriginalView') {
        if (isInTarget) {
          targetViews.push(child);
          importViews.push(child);
        }
        parseViews(view.children[i],false);
      }else {
        parseViews(view.children[i],isInTarget);
      }
    }
  }
}

/////

function nameOfSetter(propertyName) {
  if (propertyName.length>0) {
    return 'set' + (propertyName.substr(0,1)).toUpperCase() + propertyName.substr(1)+':';
  }
  return ''
}

function nameOfUpdate(propertyName) {
  if (propertyName.length>0) {
    return 'update' + (propertyName.substr(0,1)).toUpperCase() + propertyName.substr(1)+'';
  }
  return ''
}

function uiColor(color) {
  var colorName = color.name;

  if (colorName) {
    return '[UIColor '+colorName+']';
  }else {
      var colorR = Math.round(color.r*1000)/1000;
      var colorG = Math.round(color.g*1000)/1000;
      var colorB = Math.round(color.b*1000)/1000;
      var colorA = color.a;
    return '[UIColor colorWithRed:'+colorR+' green:'+colorG+' blue:'+colorB+' alpha:'+colorA+']';
  }
}

function outputAttributeLines(out,dictonaryName,paragraphStyleName,view,indentString) {
  var attribute = view.attributedText.attribute;
  var lines = out;
  var paragraphStyle = attribute.paragraphStyle;
  var dictonaryValue = attribute.value;
  /*
  var paragraphStyleName = paragraphStyle ? paragraphStyle.instance : '';
  if (useFixedVarName) {
    paragraphStyleName = 'paragraphStyle';
  }
  */
  var alignment;

  if (view.textAlignment) {
    if (view.textAlignment == 'center') {
      alignment = 'NSTextAlignmentCenter';
    }else if (view.textAlignment == 'right') {
      alignment = 'NSTextAlignmentRight';
    }
  }

  if (paragraphStyle || alignment) {
    lines.push(indentString+'NSMutableParagraphStyle *'+paragraphStyleName+' = [[NSMutableParagraphStyle alloc] init];');
    if (paragraphStyle && paragraphStyle.lineHeight) {
      var lineHeight = paragraphStyle.lineHeight;
      lines.push(indentString+paragraphStyleName+'.minimumLineHeight = '+ lineHeight +';');
      lines.push(indentString+paragraphStyleName+'.maximumLineHeight = '+ lineHeight +';');
    }

    if (alignment) {
      lines.push(indentString+paragraphStyleName+'.alignment = ' + alignment + ';');
    }
  }

  lines.push(indentString+'NSDictionary *'+dictonaryName+' = @{');
  if (paragraphStyle) {
    lines.push(indentString+'\t'+'NSParagraphStyleAttributeName : ' + paragraphStyleName +',');
  }

  if (dictonaryValue) {
    var colorAttribute = dictonaryValue.NSForegroundColorAttributeName;
    var fontAttribute = dictonaryValue.NSFontAttributeName;
    var fontAttributeName = view.font.name;
    var fontAttributeSize = view.font.size;
    var kernAttribute = dictonaryValue.NSKernAttributeName;

    if (colorAttribute) {
      lines.push(indentString+'\t'+'NSForegroundColorAttributeName : '+
      uiColor(colorAttribute)+',');
    }
    if (fontAttributeName) {
      if (view.font.name == '_system') {
        lines.push(indentString+'\t'+'NSFontAttributeName : [UIFont systemFontOfSize:'+view.font.size+ '],');
      }else if (view.font.name == '_system') {
          lines.push(indentString+'\t'+'NSFontAttributeName : [UIFont boldSystemFontOfSize:'+view.font.size+ '],');
      }else {
        lines.push(indentString+'\t'+'NSFontAttributeName : [UIFont fontWithName:@"'+view.font.name+
        '" size:'+view.font.size+ '],');
      }
    }
    if (kernAttribute) {
      lines.push(indentString+'\t'+'NSKernAttributeName : [NSNumber numberWithFloat:'+kernAttribute+ '],');
    }

  }
  lines.push(indentString+'};');
}

function output() {
  var out = []
  var startView = viewsWithIds[startViewId];
  var startClass = startView.className ? startView.className:'MyViewController';

  var selfPrefix = "self.";
  var selfViewName = "self";
  var rootViewName = "self.view";

  out.push('#import "'+startClass+'.h"');
  for(var i=0; i<importViews.length; i++) {
    out.push('#import "'+importViews[i].className+'.h"');
  }

  out.push('');
  out.push('@implementation '+startClass);
  out.push('');

  if (isViewController) {
    out.push('- (void)viewDidLoad {');
    out.push('\t[super viewDidLoad];');
    out.push('\t[self configureView];');
    out.push('}');
    out.push('');
    out.push('- (void)configureView {');
    out.push('\t'+selfPrefix+'scrollView = [UIScrollView new];');
    out.push('\t'+selfPrefix+'scrollView.frame = self.view.bounds;');

    out.push('\t'+selfPrefix+'scrollView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;');
    out.push('\t'+'['+rootViewName+' addSubview:self.scrollView];');
    out.push('');

  }else {
    out.push('- (instancetype)initWithCoder:(NSCoder *)aDecoder {');
    out.push('\tself = [super initWithCoder:aDecoder];');
    out.push('\tif (self) {');
    out.push('\t\t[self commonInit];');
    out.push('\t}');
    out.push('\treturn self;');
    out.push('}');
    out.push('');

    out.push('- (instancetype)initWithFrame:(CGRect)frame {');
    out.push('\tself = [super initWithFrame:frame];');
    out.push('\tif (self) {');
    out.push('\t\t[self commonInit];');
    out.push('\t}');
    out.push('\treturn self;');
    out.push('}');
    out.push('');
    out.push('- (void)commonInit {');
    if (targetViews.length>0) {
      out.push('\t'+selfPrefix+'bounds = CGRectMake(0, 0, '+
        targetViews[0].frame.width+', '+targetViews[0].frame.height+');');
        out.push('');
    }
  }

  for(var i=0; i<targetViews.length; i++) {
    var view = targetViews[i];
    var isViewControllerRoot = (i==0 && isViewController)?true:false
    var isOriginalViewRoot = (i==0 && !isViewController)?true:false
    var isOriginalViewInstance = (i>0 && view.writeClassType && view.writeClassType == 'OriginalView')?true:false

    var indent = '\t';
    var subject = isOriginalViewRoot? selfViewName : selfPrefix + view.instance;
    var left = indent + subject;

    if (isOriginalViewInstance) {
      out.push(left + ' = [' +view.className+ ' new];');
    } else if (isOriginalViewRoot) {
    } else if (isViewControllerRoot) {

      out.push(left+' = [UIView new];');
        /*
      if (view['contentSize']) {
        out.push(left+'.contentSize = CGSizeMake('+subject+'.frame.size.width, '+
        view.contentSize.height+');');
      }
         */
      //out.push(left+'.backgroundColor = [UIColor lightGrayColor];');

    } else if (view.type == 'View') {

      out.push(left + ' = [UIView new];');

    } else if (view.type == 'ImageView') {

      out.push(left + ' = [UIImageView new];');
      out.push(left + '.image = [UIImage imageNamed:@"'+view.image+'"];');
      //out.push(left + '.contentMode = UIViewContentModeCenter;');

    } else if (view.type == 'Button') {

      out.push(left + ' = [UIButton buttonWithType:UIButtonTypeSystem];');

      if (view.backgroundImage) {
        out.push(indent+'['+subject+' setBackgroundImage:[UIImage imageNamed:@"' +
        view.backgroundImage+'"] forState:UIControlStateNormal];');
      }
      out.push(indent+'['+subject+' addTarget:self action:@selector('+view.instance+'Tapped:) '+
      'forControlEvents:UIControlEventTouchUpInside];');

    } else if (view.type == 'Label') {

      out.push(left + ' = [UILabel new];');
      if (view.lineCount && view.lineCount>1 ) {
        out.push(left + '.numberOfLines = 0;');
      }

      // attribute
      var text = view.isNeedAttributed? view.attributedText.text : view.text;
      var textPart = 'NSLocalizedString(@"' + text +'",nil)';

      if (view.isNeedAttributed && view.isNeedAttributed == true) {
        if (!isViewController) {
          out.push(indent + selfPrefix + view.textPropertyName + ' = ' + textPart + ';');
        }else {
          var attribute = view.attributedText.attribute;
          var paragraphStyleName = attribute.paragraphStyle ? attribute.paragraphStyle.instance : '';

          outputAttributeLines(out, attribute.instance, paragraphStyleName, view,indent);
          out.push(left+'.attributedText = [[NSAttributedString alloc] initWithString:'+
          textPart+' attributes:' + attribute.instance + '];');
        }
      }else {
        if (view.textAlignment) {
          if (view.textAlignment == 'center') {
            out.push(left+'.textAlignment = NSTextAlignmentCenter;');
          }else if (view.textAlignment == 'right') {
            out.push(left+'.textAlignment = NSTextAlignmentRight;');
          }
        }

        if (view.textColor) {
          out.push(left + '.textColor = '+uiColor(view.textColor) +';');
        }
        if (view.font) {
          if (view.font.name == '_system') {
            out.push(left+'.font = [UIFont systemFontOfSize:'+view.font.size+ '];');
          }else if (view.font.name == '_systemBold') {
            out.push(left+'.font = [UIFont boldSystemFontOfSize:'+view.font.size+ '];');
          }else {
            out.push(left+'.font = [UIFont fontWithName:@"'+view.font.name+ '" size:'+view.font.size + '];');
          }
        }
        if (!isViewController) {
          out.push(indent + selfPrefix + view.textPropertyName + ' = ' + textPart + ';');
        }else {
          out.push(left + '.text = ' + textPart + ';');

        }
//        out.push(left+'.text = '+textPart+';');
      }


    } else if (view.type == 'TextField') {

      out.push(left + ' = [UITextField new];');

    } else if (view.type == 'TextView') {

      out.push(left + ' = [UITextView new];');

    }

    // view decoration
    if (!isOriginalViewRoot) {

      if (view.alpha && view.alpha<1.0) {
        out.push(left+'.alpha = '+view.alpha+';');
      }
    }
    if ((isViewController && !isOriginalViewRoot)|| (!isViewController && !isOriginalViewRoot) ) {
      if (view.frame) {
        if (view.contentSize) {
          out.push(left+'.frame = CGRectMake(0, 0, '+view.contentSize.width+', '+view.contentSize.height+');');
          //out.push(left+'.contentSize = CGSizeMake('+view.contentSize.width+', '+view.contentSize.height+');');
          out.push(left+'.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleBottomMargin;');
        }else {
          out.push(left+".frame = CGRectMake("+view.frame.x+", "+view.frame.y+", "+
          view.frame.width+", "+view.frame.height+");");
        }

        if (view.constraints) {
          var hFormat = "";
          if (!view.constraints.right) {
            hFormat = "UIViewAutoresizingFlexibleRightMargin";
          }else if (!view.constraints.width) {
            hFormat = "UIViewAutoresizingFlexibleWidth";
          }else if (!view.constraints.left) {
            hFormat = "UIViewAutoresizingFlexibleLeftMargin";
          }

          var vFormat = "";
          if (!view.constraints.bottom) {
            vFormat = "UIViewAutoresizingFlexibleBottomMargin";
          }else if (!view.constraints.height) {
            vFormat = "UIViewAutoresizingFlexibleHeight";
          }else if (!view.constraints.top) {
            vFormat = "UIViewAutoresizingFlexibleTopMargin";
          }
          out.push(left+".autoresizingMask = "+hFormat+" | "+vFormat+";");
        }
      }
      out.push('');
    }
    if (!isOriginalViewInstance) {
      if (view.borderColor) {
        out.push(left+'.layer.borderColor = ['+uiColor(view.borderColor)+' CGColor];');
      }
      if (view.borderWidth) {
        out.push(left+'.layer.borderWidth = '+view.borderWidth+';');
      }
      if (view.cornerRadius) {
        out.push(left+'.layer.cornerRadius = '+view.cornerRadius+';');
      }
      if (!(typeof view.userInteractionEnabled === "undefined") && (view.userInteractionEnabled==false)) {
        out.push(left+'.userInteractionEnabled = NO;');
      }
      if (view.backgroundColor) {
        out.push(left+'.backgroundColor = '+ uiColor(view.backgroundColor) +';');
      }
      if (view.clipsToBounds) {
        out.push(left+'.clipsToBounds = YES;');
      }
    }

    // add subview

    var parentName = '';
    if (isViewController) {
      parentName = selfPrefix+parentViewsWithIds[view.originalId]['instance'];
      if (view.originalId == startViewId) {
          parentName = "self.scrollView"; // rootViewName
      }
    }else {
      parentName = selfViewName;
      if (parentViewsWithIds[view.originalId] && startViewId!=parentViewsWithIds[view.originalId].originalId) {
          parentName = selfPrefix+parentViewsWithIds[view.originalId]['instance'];
      }
    }
    if (!isOriginalViewRoot) {
      out.push(indent + '['+parentName+' addSubview:'+subject+'];');
      out.push('');
    }

  }

  if (isViewController) {
    var rootName = viewsWithIds[startViewId].instance;

//    out.push('\t'+selfPrefix+rootName+'.contentSize = CGSizeMake('+selfPrefix+rootName+'.frame.size.width, '+selfPrefix+rootName+'.contentSize.height);');
//    out.push('\t'+selfPrefix+rootName+'.frame = CGRectMake(0, 0, self.view.bounds.size.width, self.view.bounds.size.height);');
    out.push('\t'+selfPrefix+rootName+'.frame = CGRectMake(0, 0, self.view.bounds.size.width, '+selfPrefix+rootName+'.bounds.size.height);');
    out.push('\t'+selfPrefix+'scrollView.contentSize = '+selfPrefix+rootName+'.frame.size;');

    out.push('}');
    out.push('');
  }else {
    out.push('}');
    out.push('');

  }

  if (textPropertyViews.length>0) {
    out.push('# pragma mark text properties');
    out.push('');
    for(var i=0; i<textPropertyViews.length; i++) {
      var view = textPropertyViews[i];
      var textPropertyName = view.textPropertyName;
      var name = view.instance;

      out.push('- (void)'+nameOfSetter(textPropertyName) + '(NSString*)text {');
      if (view.isNeedAttributed && view.isNeedAttributed == true) {

        out.push('\t' + 'if (![_' + textPropertyName + ' isEqualToString:text]) {');
        out.push('\t\t' + '_' + textPropertyName + ' = [text copy];');
        out.push('\t\t' + '[self '+nameOfUpdate(name)+'];');
        out.push('\t' + '}');
        out.push('}');
        out.push('');
        out.push('- (void)' + nameOfUpdate(name) + ' {');

        var attribute = view.attributedText.attribute;
        var paragraphStyleName = attribute.paragraphStyle ? attribute.paragraphStyle.instance : '';

        out.push('\t' + 'NSString *text = ' + selfPrefix + textPropertyName + ' ? ' + selfPrefix + textPropertyName + ' : @"";');
        outputAttributeLines(out, "attribute", "paragraphStyle", view,indent);

        out.push('\t'+selfPrefix+name+'.attributedText = [[NSAttributedString alloc] initWithString:text attributes:' + 'attribute' + '];');

        out.push('}');
        //        footerLines = footerLines.concat( textSettingLines(view, name, 'text',  isNeedAttributed, '\t' , true) );
      } else {
        out.push('\t' + 'if (![_' + textPropertyName + ' isEqualToString:text]) {');
        out.push('\t\t' + '_' + textPropertyName + ' = [text copy];');
        out.push('\t\t'+selfPrefix + name +'.text = '+ '_' + textPropertyName + ';');
        out.push('\t' + '}');
        out.push('}');
      }
      out.push('');
    }

  }

  if (imagePropertyViews.length>0) {
    out.push('# pragma mark image properties');
    out.push('');
    for(var i=0; i<imagePropertyViews.length; i++) {
      var view = imagePropertyViews[i];
      var imagePropertyName = view.imagePropertyName;
      var name = view.instance;

      out.push('- (void)'+nameOfSetter(imagePropertyName) + '(UIImage*)image {');
      out.push('\t' + '_' + imagePropertyName + ' = image;');
      out.push('\t'+selfPrefix + name +'.image = image;');
      out.push('}');
      out.push('');
    }

  }

  out.push('@end');
  out.push('');

  return out.join('\n');
}
